/* ================================================================= 
   file    : printfile.p
   dd      : august 16, 1998
   purpose : print a text file
   by      : Jurjen Dijkstra
             mailto:jurjen@global-shared.com
             see also http://www.global-shared.com/api
           : Nickolay Borshukov
             mailto:nborshukov@yahoo.com     
   usage   : RUN printfile.p ("printfile.p").
   tested  : only on Windows 98 with local printer (HP Laserjet 4)
             and on Windows NT 4.0 with network printer (HP Lasejet 4000)    
   ================================================================= */
DEFINE INPUT PARAMETER p-Filename    AS CHAR NO-UNDO.                 /* file name of file to print           */
DEFINE INPUT PARAMETER p-Devicename  AS CHAR    NO-UNDO.              /* if "" then default printer
                                                                         if undefined then run setup dialog   */
DEFINE INPUT PARAMETER p-docname     AS CHAR    NO-UNDO.              /* if "" then p-Filename                */
DEFINE INPUT PARAMETER p-Landscape   AS LOGICAL NO-UNDO INITIAL No.   /* Orientation is Landscape or Portrait */
DEFINE INPUT PARAMETER p-Leftmargin  AS DECIMAL NO-UNDO INITIAL 2.    /* left margin (cm) from paper edge     */
DEFINE INPUT PARAMETER p-Rightmargin AS DECIMAL NO-UNDO INITIAL 1.5.  /* right margin (cm) from paper edge    */
DEFINE INPUT PARAMETER p-fontsize    AS INTEGER NO-UNDO INITIAL 9.    /* pointsize for bodyfont               */
DEFINE INPUT PARAMETER print-h-f     AS LOGICAL NO-UNDO INITIAL TRUE. /* Print header and footer              */
DEFINE INPUT PARAMETER print-code    AS LOGICAL NO-UNDO INITIAL False. /* Print code                          */


/* these variables could also be useful as input parameters: */
DEF VAR p-Silent      AS LOGICAL NO-UNDO INITIAL No.  /* show no message boxes (on error)     */

/* ------ status strings. to be checked by calling program ---------- */

  &GLOB status_nodefaultdevice            "error 1: there is no default printer installed on this system"
  &GLOB status_filenotfound    SUBSTITUTE("error 2: file &1 not found", p-Filename)
  &GLOB status_CreateDC        SUBSTITUTE("error 3: can not create context for device '&1'", p-Devicename)
  &GLOB status_StartDoc                   "error 4: can not create print job"
  &GLOB status_ok                         ""

/* ------ jump to main procedure ------------------------------------ */
/* NIKI begin */ 

&GLOB num-brakes 20

DEF VAR MarginRightPix   AS INTEGER NO-UNDO.
DEF VAR TextWidth        AS INTEGER NO-UNDO.
DEF VAR LeadingSpaces    AS INTEGER NO-UNDO.
DEF VAR A-Width          AS INTEGER NO-UNDO.
DEF VAR num-copies       AS INTEGER NO-UNDO initial 1.
def var is-ok            as logical no-undo.
def var max-breaks       as integer no-undo initial {&num-brakes}. 
def var body-char-width  as integer no-undo.
def var print-mode       as integer no-undo initial 1. /* 1 - normal; 2 paged i.e. Excel type */ 
def var column-no        as integer no-undo initial 1.
def var PaperSize        as integer no-undo initial 9. /* 1-letter; 8-A3; 9-A4 */

    if p-Devicename = ? then do:
        p-Devicename = "".
        run printfile-ss.w(input p-Filename,
                           input p-docname, 
                          input-output p-Devicename,
                          input-output PaperSize,
                          input-output p-Landscape,
                          input-output p-Leftmargin,
                          input-output p-Rightmargin,
                          input-output p-fontsize,
                          input-output print-h-f,
                          input-output print-code,
                          input-output num-copies,
                          output is-ok).
        if not is-ok then return.
        if print-code = ? then do:
            print-mode = 2.
            print-code = false.
        end.
    end. 
           
    p-LeftMargin = p-LeftMargin / 2.54.
    p-RightMargin = p-RightMargin / 2.54.

/* NIKI end */
  session:set-wait-state("general").  
  RUN Main.
  session:set-wait-state("").
  IF (NOT p-Silent) AND (RETURN-VALUE<>{&status_ok}) THEN
     MESSAGE RETURN-VALUE VIEW-AS ALERT-BOX ERROR.
  RETURN RETURN-VALUE.

/* ------ WIN32 API definitions   ----------------------------------- */

/* moved to includefile for better readability */
{printfile.i}

/* ------ misc variables -------------------------------------------- */

  DEF VAR chStatus         AS CHAR    NO-UNDO.
  DEF VAR ReturnValue      AS INTEGER NO-UNDO.
  DEF VAR hPrinterDC       AS INTEGER NO-UNDO.
  DEF VAR CurrentY         AS INTEGER NO-UNDO.
  DEF VAR LineHeight       AS INTEGER NO-UNDO.
  DEF VAR PageHeight       AS INTEGER NO-UNDO.
  DEF VAR PageWidth        AS INTEGER NO-UNDO.
  DEF VAR HeaderHeight     AS INTEGER NO-UNDO.
  DEF VAR FooterHeight     AS INTEGER NO-UNDO.
  DEF VAR PageNumber       AS INTEGER NO-UNDO.
  DEF VAR textline         AS CHAR    NO-UNDO.
  DEF VAR PageStarted      AS LOGICAL NO-UNDO.
  DEF VAR MarginLeftPix    AS INTEGER NO-UNDO.
    
  DEF VAR hBodyFont        AS INTEGER NO-UNDO.
  DEF VAR hHeaderFont      AS INTEGER NO-UNDO.
  DEF VAR hStockFont       AS INTEGER NO-UNDO.
  DEF VAR HeaderFontHeight AS INTEGER NO-UNDO.

  DEF STREAM textfile.

/* ------ main procedure -------------------------------------------- */

PROCEDURE Main :
    
DEF VAR i                  AS INTEGER NO-UNDO.    
DEF VAR full-line          as character no-undo.

  FILE-INFO:FILE-NAME = p-Filename.
  IF FILE-INFO:FULL-PATHNAME=? THEN
    RETURN {&status_filenotfound}.
  RUN StartDocument.
  IF chStatus <> "" THEN DO:
     RUN EndDocument.
     RETURN chStatus.
  END.
  RUN GetPrinterInfo.

  case print-mode:
    when 1 then do:
        INPUT STREAM textfile FROM VALUE(p-Filename).
          REPEAT:
             full-line = "".
             IMPORT STREAM textfile UNFORMATTED full-line.
             if full-line <> "" then do:
                  LeadingSpaces = 0.
                  repeat:
                      if substring(full-line,LeadingSpaces + 1,1) <> chr(32) then leave.
                      LeadingSpaces = LeadingSpaces + 1. 
                  end.
      /* next lines are added to prevent large number of leading spaces */
                  run get-line-width(input chr(32), output A-Width).
                  if LeadingSpaces > truncate(0.6 * TextWidth / A-Width,0) then do:
                      LeadingSpaces = truncate(0.6 * TextWidth / A-Width,0).
                      full-line = fill(chr(32),LeadingSpaces) + trim(full-line).
                  end.    
      /* END lines */
                  max-breaks = {&num-brakes}.     
                  run break-full-line(input-output full-line).     
                  do i = 1 to num-entries(full-line,chr(10)):
                      textline = entry(i,full-line,chr(10)).
                      RUN PrintTextLine.
                  end.
             end.     
             else do:         
                  textline = "".
                  RUN PrintTextLine.
             end.
          END.
        INPUT STREAM textfile CLOSE.
    end. /* when 1 */
    when 2 then do:
        def var chars-per-line as integer no-undo.
        def var is-next-column as logical initial true no-undo.
        def var start-pos as integer no-undo.
      
        chars-per-line = truncate(TextWidth / body-char-width,0).
        do while is-next-column:  
          is-next-column = false.
          start-pos = 1 + (column-no - 1) * chars-per-line.
          INPUT STREAM textfile FROM VALUE(p-Filename).
            REPEAT:
                import STREAM textfile unformatted full-line.
                if length(full-line) > column-no * chars-per-line 
                      then is-next-column = true.
                textline = substring(full-line,start-pos,chars-per-line).
                RUN PrintTextLine.
            END.
          INPUT STREAM textfile CLOSE.
          column-no = column-no + 1.
          PageNumber = 0.
          if is-next-column 
                then run NewPage.
                
        end.  
    end. /* when 2 */
  end case.  
  RUN EndDocument.

  RETURN {&status_ok}.

END PROCEDURE.

/* ------ private internal procedures ------------------------------- */
PROCEDURE break-full-line:
define input-output parameter f-line as character no-undo.

def var rest-line          as character no-undo.
DEF VAR cur-line-width     AS INTEGER NO-UNDO.
DEF VAR char-width         AS INTEGER NO-UNDO.
def var num-chars          as integer no-undo.
def var delim-pos          as integer no-undo.
def var i                  as integer no-undo.
def var mode               as integer no-undo.

    max-breaks = max-breaks - 1.
    if max-breaks < 0 then do:
        message "Error braking line:" skip f-line view-as alert-box error.
        return.
    end.    
    rest-line = entry(num-entries(f-line,chr(10)),f-line,chr(10)).
    if print-code and num-entries(f-line,chr(10)) > 1 then rest-line = fill(chr(32),LeadingSpaces + 4) + rest-line.
    
    run get-line-width(input rest-line, output cur-line-width).
    if cur-line-width LE TextWidth then do:
        entry(num-entries(f-line,chr(10)),f-line,chr(10)) = rest-line.
        return.
    end.    
    if num-entries(trim(rest-line),chr(32)) = 1 then do:
        mode = 1.
    end.
    else do:
        run get-line-width(input entry(1,rest-line,chr(32)), output cur-line-width).
        if cur-line-width LE TextWidth 
                    and trim(entry(1,rest-line,chr(32))) <> ""
            then do:
                mode = 2.
            end.    
            else do:
                mode = 1.
            end.
    end.    
    case mode:
        when 1 then do:
            run get-line-width(input "A", output char-width).
            num-chars = truncate(TextWidth / char-width,0).
            repeat:
                run get-line-width(input substring(rest-line,1,num-chars), output cur-line-width).
                if cur-line-width LE TextWidth then do:
                    entry(num-entries(f-line,chr(10)),f-line,chr(10)) = substring(rest-line,1,num-chars) + chr(10) + substring(rest-line,num-chars + 1).                                
                    leave.
                end.
                else do:
                    num-chars = num-chars - 1.
                end.
            end.
            run break-full-line(input-output f-line).
        end.
        when 2 then do:
            i = 1.
            num-chars = 1.
            repeat:
                i = index(rest-line,chr(32),i).
                run get-line-width(input substring(rest-line,1,i), output cur-line-width).
                if cur-line-width GT TextWidth or i = 0 then do:
                    entry(num-entries(f-line,chr(10)),f-line,chr(10)) = substring(rest-line,1,num-chars) + chr(10) + substring(rest-line,num-chars + 1).
                    leave.
                end.
                else do:
                    num-chars = i.
                    i = i + 1.
                end.
            end.
            run break-full-line(input-output f-line).
        end.
        otherwise return.
    end case.
    

END PROCEDURE.

PROCEDURE get-line-width:
define input parameter c        as character  no-undo.
define output parameter wid     as integer no-undo.
    
def var pSize   as memptr no-undo.
def var res     as integer no-undo.
    
    set-size(pSize) = 8.
    run GetTextExtentPoint32A(hPrinterDC,c,length(c),pSize,output res).
    if res <> 0 then wid = get-long(pSize,1).
    set-size(pSize) = 0.

END PROCEDURE.

PROCEDURE PrintTextLine :

  IF (NOT PageStarted) 
     OR (CurrentY + LineHeight > PageHeight - FooterHeight) THEN
     RUN NewPage.

  RUN TextOutA( hPrinterDC,
                MarginLeftPix, 
                CurrentY, 
                textline, 
                LENGTH(textline), 
                OUTPUT ReturnValue).
  CurrentY = CurrentY + LineHeight.

END PROCEDURE.


PROCEDURE NewPage :

  IF PageStarted THEN DO:
     RUN EndPage (hPrinterDC, OUTPUT ReturnValue).
     PageStarted = NO.
  END.

  IF NOT PageStarted THEN DO:
     RUN StartPage (INPUT hPrinterDC, OUTPUT ReturnValue).
     PageStarted = YES.
     PageNumber  = PageNumber + 1.
     if print-h-f then do:
        RUN PrintHeader.
        RUN PrintFooter.
     end.   
     IF hBodyFont<>0 THEN
        RUN SelectObject(hPrinterDC, hBodyFont, OUTPUT ReturnValue).
     RUN SetTextAlign(hPrinterDC, {&TA_TOP} + {&TA_LEFT}, OUTPUT ReturnValue).
     CurrentY = HeaderHeight.
  END.

END PROCEDURE.

PROCEDURE PrintHeader :

  IF hHeaderFont<>0 THEN
     RUN SelectObject(hPrinterDC, hHeaderFont, OUTPUT ReturnValue).

  RUN PrintInformation ( 0,
                         {&TA_TOP},
                         "",
                         "",
                         p-docname).
  RUN HorizontalLine(HeaderFontHeight + 5).
  HeaderHeight = HeaderFontHeight + 10.

END PROCEDURE.

PROCEDURE PrintFooter :

  IF hHeaderFont<>0 THEN
     RUN SelectObject(hPrinterDC, hHeaderFont, OUTPUT ReturnValue).

  RUN PrintInformation ( PageHeight,
                         {&TA_BOTTOM},
                         ' ' + STRING(TODAY, "99-99-9999"),
                         ' ' + STRING(TIME, "HH:MM:SS"),
                         '. ' + STRING(PageNumber) + (if print-mode = 2 then "." + string(column-no) else "" )).
  RUN HorizontalLine(PageHeight - HeaderFontHeight - 7).
  FooterHeight = HeaderFontHeight + 12.

END PROCEDURE.

PROCEDURE PrintInformation :
  DEFINE INPUT PARAMETER Y           AS INTEGER NO-UNDO.
  DEFINE INPUT PARAMETER valign      AS INTEGER NO-UNDO.
  DEFINE INPUT PARAMETER Left-Text   AS CHAR    NO-UNDO.
  DEFINE INPUT PARAMETER Center-Text AS CHAR    NO-UNDO.
  DEFINE INPUT PARAMETER Right-Text  AS CHAR    NO-UNDO.

  IF NOT (left-text="" or left-text=?) THEN DO:
     RUN SetTextAlign(hPrinterDC, valign + {&TA_LEFT}, OUTPUT ReturnValue).
     RUN TextOutA( hPrinterDC,
                   MarginLeftPix,
                   Y,
                   left-text,
                   LENGTH(left-text),
                   OUTPUT ReturnValue).
  END.

  IF NOT (center-text="" or center-text=?) THEN DO:
     RUN SetTextAlign(hPrinterDC, valign + {&TA_CENTER}, OUTPUT ReturnValue).
     RUN TextOutA( hPrinterDC,
                   INTEGER((MarginLeftPix + PageWidth) / 2),
                   Y,
                   center-text,
                   LENGTH(center-text),
                   OUTPUT ReturnValue).
  END.

  IF NOT (right-text="" or right-text=?) THEN DO:
     RUN SetTextAlign(hPrinterDC, valign + {&TA_RIGHT}, OUTPUT ReturnValue).
     RUN TextOutA( hPrinterDC,
                   PageWidth,
                   Y,
                   right-text,
                   LENGTH(right-text),
                   OUTPUT ReturnValue).
  END.

END PROCEDURE.

PROCEDURE HorizontalLine :
   DEFINE INPUT PARAMETER Y AS INTEGER NO-UNDO.
   RUN MoveToEx(hPrinterDC, MarginLeftPix, Y, 0, OUTPUT ReturnValue).
   RUN LineTo  (hPrinterDC, PageWidth, Y, OUTPUT ReturnValue).
END PROCEDURE.

PROCEDURE StartDocument :

    DEF VAR devicespecs      AS CHAR    NO-UNDO.
    DEF VAR lpdocname        AS MEMPTR  NO-UNDO.
    DEF VAR lpdocinfo        AS MEMPTR  NO-UNDO.
    DEF VAR JobID            AS INTEGER NO-UNDO.
    DEF VAR lpDefaultDevmode AS MEMPTR  NO-UNDO.
    DEF VAR lpDevmode        AS MEMPTR  NO-UNDO.


    /* no devicename specified? Then use default printer */
    IF p-devicename = "" THEN DO:
       devicespecs = FILL("x", 255).
       RUN GetProfileStringA ("windows":U,
                              "device":U,
                              "-unknown-,,":U,
                              OUTPUT devicespecs,
                              LENGTH(devicespecs),
                              OUTPUT ReturnValue).
       p-devicename = ENTRY(1, devicespecs).
    END.
    IF p-devicename="-unknown-":U THEN DO:
       chStatus = {&status_nodefaultdevice}.
       RETURN.
    END.
    
    IF p-docname="" or p-docname=? THEN
       p-docname=p-Filename.

    SET-SIZE(lpdocname)     = LENGTH(p-docname) + 1.
    PUT-STRING(lpdocname,1) = p-docname.
    SET-SIZE(lpdocinfo)     = 12.
    PUT-LONG(lpdocinfo, 1)  = GET-SIZE(lpdocinfo).
    PUT-LONG(lpdocinfo, 5)  = GET-POINTER-VALUE(lpdocname).
    PUT-LONG(lpdocinfo, 9)  = 0.


    /* ask for the size of the devmode structure: */
    RUN DocumentPropertiesA (0, 0,
                                p-devicename,
                                0,
                                0,
                                0,
                                output ReturnValue).
    SET-SIZE(lpDefaultDevMode) = ReturnValue.
    SET-SIZE(lpDevMode)        = ReturnValue.


    /* get the default contents of the devmode structure: */
    RUN DocumentPropertiesA (0, 0,
                                p-devicename,
                                GET-POINTER-VALUE(lpDefaultDevMode),
                                0,
                                {&DM_OUT_BUFFER},
                                output ReturnValue).
    /* specify what you want to change: */
    PUT-LONG (lpDefaultDevMode, {&CCHDEVICENAME} +  9) = {&DM_ORIENTATION} + {&DM_COPIES}.
    PUT-SHORT(lpDefaultDevMode, {&CCHDEVICENAME} + 23) = num-copies.

/* for size */
    PUT-LONG (lpDefaultDevMode, {&CCHDEVICENAME} +  9) = GET-LONG (lpDefaultDevMode, {&CCHDEVICENAME} +  9) +
                                                         {&DM_PAPERSIZE}.
    PUT-SHORT(lpDefaultDevMode, {&CCHDEVICENAME} + 15) = PaperSize. /* PAPER size  1-letter; 8-A3; 9-A4 */
/* end for size */    

    IF p-Landscape THEN
       PUT-SHORT(lpDefaultDevMode, {&CCHDEVICENAME} + 13) = {&DMORIENT_LANDSCAPE}.
    ELSE
       PUT-SHORT(lpDefaultDevMode, {&CCHDEVICENAME} + 13) = {&DMORIENT_PORTRAIT}.


    /* create a merged devmode: */
    RUN DocumentPropertiesA (0, 0,
                                p-devicename,
                                GET-POINTER-VALUE(lpDevMode),
                                GET-POINTER-VALUE(lpDefaultDevMode),
                                {&DM_IN_BUFFER} + {&DM_OUT_BUFFER},
                                output ReturnValue).

    RUN CreateDCA ("WINSPOOL":U,
                   p-devicename,
                   0,
                   GET-POINTER-VALUE(lpDevMode),
                   OUTPUT hPrinterDC).

    SET-SIZE(lpDefaultDevMode)    = 0.
    SET-SIZE(lpDevMode) = 0.


    IF hPrinterDC = 0 THEN
       chStatus = {&status_CreateDC}.
    ELSE DO:
       RUN StartDocA(INPUT hPrinterDC,
                     GET-POINTER-VALUE(lpdocinfo), 
                     OUTPUT JobID).
       IF JobID<1 THEN
       chStatus = {&status_StartDoc}.
    END.

    PageStarted         = NO.
    PageNumber          = 0.
    SET-SIZE(lpdocinfo) = 0.
    SET-SIZE(lpdocname) = 0.

END PROCEDURE.

PROCEDURE EndDocument :

    IF PageStarted THEN DO:
       RUN EndPage  (hPrinterDC, OUTPUT ReturnValue).
       PageStarted = NO.
    END.
    IF hBodyFont<>0 THEN DO:
       RUN SelectObject(hPrinterDC, hStockFont, OUTPUT ReturnValue).
       RUN DeleteObject(hBodyFont, OUTPUT ReturnValue).
    END.
    IF hHeaderFont<>0 THEN
       RUN DeleteObject(hHeaderFont, OUTPUT ReturnValue).
    RUN EndDoc   (hPrinterDC, OUTPUT ReturnValue).
    RUN DeleteDC (hPrinterDC, OUTPUT ReturnValue).
    hPrinterDC = 0.

END PROCEDURE.

PROCEDURE GetPrinterInfo :

  DEF VAR devcap AS INTEGER NO-UNDO.

  /* create font for body with specified size */
  DEF VAR emheight AS INTEGER NO-UNDO.
  RUN GetDeviceCaps (hPrinterDC, {&LOGPIXELSY}, OUTPUT devcap).
  RUN MulDiv (p-fontsize, devcap, 72, OUTPUT emheight).
  emheight = 0 - emheight.
  RUN CreateFontA ( emheight,
                    0,   /* width (0=default)                       */
                    0,   /* escapement                              */
                    0,   /* orientation                             */
                    400, /* weight: LIGHT=300, NORMAL=400, BOLD=700 */
                    0,   /* italics                                 */
                    0,   /* underline                               */
                    0,   /* strikeout                               */
                    {&ANSI_CHARSET},
                    {&OUT_DEFAULT_PRECIS},
                    {&CLIP_DEFAULT_PRECIS},
                    {&PROOF_QUALITY},
                    {&FIXED_PITCH} + {&FF_DONTCARE},
                    "Courier New Cyr":U,
                    OUTPUT hBodyFont).
  IF hBodyFont<>0 THEN
     RUN SelectObject(hPrinterDC, hBodyFont, OUTPUT hStockFont).
  RUN GetLineHeight(OUTPUT LineHeight).
  RUN GetBodyCharWidth(OUTPUT body-char-width).
    
  /* create font for header and footer */
  RUN GetDeviceCaps (hPrinterDC, {&LOGPIXELSY}, OUTPUT devcap).
  RUN MulDiv (7, devcap, 72, OUTPUT emheight).
  emheight = 0 - emheight.
  RUN CreateFontA ( emheight,
                    0,
                    0,
                    0,
                    400,
                    1,
                    0,
                    0,
                    {&ANSI_CHARSET},
                    {&OUT_DEFAULT_PRECIS},
                    {&CLIP_DEFAULT_PRECIS},
                    {&PROOF_QUALITY},
                    {&VARIABLE_PITCH} + {&FF_DONTCARE},
                    "Arial Cyr":U,
                    OUTPUT hHeaderFont).
  IF hHeaderFont<>0 THEN DO:
     RUN SelectObject(hPrinterDC, hHeaderFont, OUTPUT ReturnValue).
     RUN GetLineHeight(OUTPUT HeaderFontHeight).
  END.
  ELSE 
     ASSIGN 
        hHeaderFont      = hBodyFont
        HeaderFontHeight = LineHeight.

  /* get the left margin, in pixels */
  
  RUN GetDeviceCaps (hPrinterDC, {&LOGPIXELSX}, OUTPUT devcap).
  MarginLeftPix = p-Leftmargin * devcap.
  MarginRightPix = p-Rightmargin * devcap.
  
  RUN GetDeviceCaps (hPrinterDC, {&PHYSICALOFFSETX}, OUTPUT devcap).
  MarginLeftPix = MAXIMUM(MarginLeftPix - devcap, 0).

def var right-offset as integer no-undo. /* right "white" field */

  right-offset = devcap. /* left-offset */    
  RUN GetDeviceCaps (hPrinterDC, {&PHYSICALWIDTH}, OUTPUT devcap).
  right-offset = devcap - right-offset. /* physical-wid - left-offset */

  /* get the page height (well, the printable part) */
  RUN GetDeviceCaps (hPrinterDC, {&VERTRES}, OUTPUT PageHeight).
  /* get the page width (the printable part) */
  RUN GetDeviceCaps (hPrinterDC, {&HORZRES}, OUTPUT PageWidth).
  right-offset = right-offset - PageWidth. /* physical-wid - left-offset - printable-widht */
    
  MarginRightPix = MAXIMUM(MarginRightPix - right-offset, 0).

  PageWidth = PageWidth - MarginRightPix.   
  TextWidth = PageWidth - MarginLeftPix.  

END PROCEDURE.


PROCEDURE GetLineHeight :
    DEFINE OUTPUT PARAMETER p-LH AS INTEGER.

    DEF VAR lpTm AS MEMPTR NO-UNDO.
    SET-SIZE(lpTM)= 15 * 4 + 5.
    RUN GetTextMetricsA (INPUT  hPrinterDC,
                         INPUT  GET-POINTER-VALUE(lpTM),
                         OUTPUT ReturnValue).
    p-LH = GET-LONG(lpTM, 1).
    SET-SIZE(lpTM)=0.

END PROCEDURE.

PROCEDURE GetBodyCharWidth :
    DEFINE OUTPUT PARAMETER p-LH AS INTEGER.

    DEF VAR lpTm AS MEMPTR NO-UNDO.
    SET-SIZE(lpTM)= 15 * 4 + 5.
    RUN GetTextMetricsA (INPUT  hPrinterDC,
                         INPUT  GET-POINTER-VALUE(lpTM),
                         OUTPUT ReturnValue).
    p-LH = GET-LONG(lpTM, 21).
    SET-SIZE(lpTM)=0.
END PROCEDURE.
/* eof */
